#include "TFT_Backlight_objects.h"
#include "TFT_Backlight_resources.h"
#include "built_in.h"


// TFT module connections
char TFT_DataPort at PORTJ;
sbit TFT_RST at LATD4_bit;
sbit TFT_BLED at LATH5_bit;
sbit TFT_RS at LATH6_bit;
sbit TFT_CS at LATG3_bit;
sbit TFT_RD at LATH1_bit;
sbit TFT_WR at LATH2_bit;
char TFT_DataPort_Direction at TRISJ;
sbit TFT_RST_Direction at TRISD4_bit;
sbit TFT_BLED_Direction at TRISH5_bit;
sbit TFT_RS_Direction at TRISH6_bit;
sbit TFT_CS_Direction at TRISG3_bit;
sbit TFT_RD_Direction at TRISH1_bit;
sbit TFT_WR_Direction at TRISH2_bit;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at LATA0_bit;
sbit DriveX_Right at LATD7_bit;
sbit DriveY_Up at LATA1_bit;
sbit DriveY_Down at LATD4_bit;
sbit DriveX_Left_Direction at TRISA0_bit;
sbit DriveX_Right_Direction at TRISD7_bit;
sbit DriveY_Up_Direction at TRISA1_bit;
sbit DriveY_Down_Direction at TRISD4_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 800;
char PenDown;
typedef unsigned long TPointer;
TPointer PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TCButton *local_cbutton;
TCButton *exec_cbutton;
int cbutton_order;
TCButton_Round *local_round_cbutton;
TCButton_Round *exec_round_cbutton;
int round_cbutton_order;
TCLabel *local_clabel;
TCLabel *exec_clabel;
int clabel_order;
TProgressBar *local_progress_bar;
TProgressBar *exec_progress_bar;
int progress_bar_order;


void Init_ADC() {
  ADC_Init();
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 0, 1);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TCButton_Round          Button1 = 
         {
         &Screen1              , //   Button1.OwnerScreen
         0                     , //   Button1.Order           
         168                   , //   Button1.Left            
         50                    , //   Button1.Top              
         60                    , //   Button1.Width            
         60                    , //   Button1.Height        
         1                     , //   Button1.Pen_Width          
         0x0000                , //   Button1.Pen_Color       
         1                     , //   Button1.Visible         
         1                     , //   Button1.Active           
         1                     , //   Button1.Transparent     
         &Button1_Caption      , //   Button1.Caption        
         _taCenter             , //   Button1.TextAlign        
         &Tahoma19x19_Bold     , //   Button1.FontName        
         0x0000                , //   Button1.FontColor       
         1                     , //   Button1.Gradient        
         0                     , //   Button1.Gradient_Orientation    
         0xFFFF                , //   Button1.Gradient_Start_Color    
         0xC618                , //   Button1.Gradient_End_Color    
         0xC618                , //   Button1.Color           
         3                     , //   Button1.CornerRadius           
         1                     , //   Button1.PressColEnabled 
         0xE71C                , //   Button1.Press_Color     
         0                     , //   Button1.OnUpPtr
         0                     , //   Button1.OnDownPtr
         ButtonUpClick         , //   Button1.OnClickPtr
         0                       //   Button1.OnPressPtr
         };
const char Button1_Caption[2] = "+";

  TCButton_Round          Button2 = 
         {
         &Screen1              , //   Button2.OwnerScreen
         1                     , //   Button2.Order           
         250                   , //   Button2.Left            
         50                    , //   Button2.Top              
         60                    , //   Button2.Width            
         60                    , //   Button2.Height        
         1                     , //   Button2.Pen_Width          
         0x0000                , //   Button2.Pen_Color       
         1                     , //   Button2.Visible         
         1                     , //   Button2.Active           
         1                     , //   Button2.Transparent     
         &Button2_Caption      , //   Button2.Caption        
         _taCenter             , //   Button2.TextAlign        
         &Tahoma19x19_Bold     , //   Button2.FontName        
         0x0000                , //   Button2.FontColor       
         1                     , //   Button2.Gradient        
         0                     , //   Button2.Gradient_Orientation    
         0xFFFF                , //   Button2.Gradient_Start_Color    
         0xC618                , //   Button2.Gradient_End_Color    
         0xC618                , //   Button2.Color           
         3                     , //   Button2.CornerRadius           
         1                     , //   Button2.PressColEnabled 
         0xE71C                , //   Button2.Press_Color     
         0                     , //   Button2.OnUpPtr
         0                     , //   Button2.OnDownPtr
         ButtonDwnClick        , //   Button2.OnClickPtr
         0                       //   Button2.OnPressPtr
         };
const char Button2_Caption[2] = "-";

  TProgressBar                 ProgressBar1;
char ProgressBar1_Caption[5] = "50\%";

  TCButton_Round          Button3 = 
         {
         &Screen1              , //   Button3.OwnerScreen
         3                     , //   Button3.Order           
         60                    , //   Button3.Left            
         140                   , //   Button3.Top              
         201                   , //   Button3.Width            
         84                    , //   Button3.Height        
         1                     , //   Button3.Pen_Width          
         0x0000                , //   Button3.Pen_Color       
         1                     , //   Button3.Visible         
         1                     , //   Button3.Active           
         1                     , //   Button3.Transparent     
         &Button3_Caption      , //   Button3.Caption        
         _taCenter             , //   Button3.TextAlign        
         &Tahoma19x19_Bold     , //   Button3.FontName        
         0x0000                , //   Button3.FontColor       
         1                     , //   Button3.Gradient        
         0                     , //   Button3.Gradient_Orientation    
         0xFFFF                , //   Button3.Gradient_Start_Color    
         0xC618                , //   Button3.Gradient_End_Color    
         0xC618                , //   Button3.Color           
         3                     , //   Button3.CornerRadius           
         1                     , //   Button3.PressColEnabled 
         0xE71C                , //   Button3.Press_Color     
         0                     , //   Button3.OnUpPtr
         0                     , //   Button3.OnDownPtr
         Button3OnClick        , //   Button3.OnClickPtr
         0                       //   Button3.OnPressPtr
         };
const char Button3_Caption[23] = "screen & backlight off";

  TCLabel          Label1 = 
         {
         &Screen1              , //   Label1.OwnerScreen
         4                     , //   Label1.Order          
         70                    , //   Label1.Left           
         16                    , //   Label1.Top             
         192                   , //   Label1.Width         
         21                    , //   Label1.Height       
         &Label1_Caption       , //   Label1.Caption        
         &Tahoma19x19_Bold     , //   Label1.FontName       
         0x0000                , //   Label1.Font_Color     
         1                     , //   Label1.Visible        
         0                     , //   Label1.Active        
         0                     , //   Label1.OnUpPtr
         0                     , //   Label1.OnDownPtr
         0                     , //   Label1.OnClickPtr
         0                       //   Label1.OnPressPtr
         };
const char Label1_Caption[23] = "Backlight control demo";

  TCButton_Round         * const code Screen1_CButtons_Round[3]=
         {
         &Button1,             
         &Button2,             
         &Button3              
         };
  TCLabel                * const code Screen1_CLabels[1]=
         {
         &Label1               
         };
  TProgressBar           * const code Screen1_ProgressBars[1]=
         {
         &ProgressBar1         
         };

  TScreen                Screen2;
  TCButton                Button4 = 
         {
         &Screen2              , //   Button4.OwnerScreen
         0                     , //   Button4.Order           
         3                     , //   Button4.Left            
         5                     , //   Button4.Top              
         312                   , //   Button4.Width            
         231                   , //   Button4.Height        
         1                     , //   Button4.Pen_Width          
         0x0000                , //   Button4.Pen_Color       
         0                     , //   Button4.Visible         
         1                     , //   Button4.Active           
         1                     , //   Button4.Transparent     
         &Button4_Caption      , //   Button4.Caption        
         _taCenter             , //   Button4.TextAlign        
         &Tahoma19x19_Bold     , //   Button4.FontName        
         0x0000                , //   Button4.FontColor       
         0                     , //   Button4.Gradient        
         0                     , //   Button4.Gradient_Orientation    
         0xFFFF                , //   Button4.Gradient_Start_Color    
         0xC618                , //   Button4.Gradient_End_Color    
         0x0000                , //   Button4.Color           
         0                     , //   Button4.PressColEnabled 
         0xE71C                , //   Button4.Press_Color     
         0                     , //   Button4.OnUpPtr
         0                     , //   Button4.OnDownPtr
         Button4OnClick        , //   Button4.OnClickPtr
         0                       //   Button4.OnPressPtr
         };
const char Button4_Caption[3] = "on";

  TCButton               * const code Screen2_CButtons[1]=
         {
         &Button4              
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.CButtonsCount             = 0;
  Screen1.CButtons_RoundCount       = 3;
  Screen1.CButtons_Round            = Screen1_CButtons_Round;
  Screen1.CLabelsCount              = 1;
  Screen1.CLabels                   = Screen1_CLabels;
  Screen1.ProgressBarsCount         = 1;
  Screen1.ProgressBars              = Screen1_ProgressBars;
  Screen1.ObjectsCount              = 5;

  Screen2.Color                     = 0x0000;
  Screen2.Width                     = 320;
  Screen2.Height                    = 240;
  Screen2.CButtonsCount             = 1;
  Screen2.CButtons                  = Screen2_CButtons;
  Screen2.CButtons_RoundCount       = 0;
  Screen2.CLabelsCount              = 0;
  Screen2.ProgressBarsCount         = 0;
  Screen2.ObjectsCount              = 1;


  ProgressBar1.OwnerScreen     = &Screen1;
  ProgressBar1.Order           = 2;
  ProgressBar1.Left            = 14;
  ProgressBar1.Top             = 50;
  ProgressBar1.Width           = 132;
  ProgressBar1.Height          = 60;
  ProgressBar1.Pen_Width       = 1;
  ProgressBar1.Pen_Color       = 0x0000;
  ProgressBar1.Visible         = 1;
  ProgressBar1.Transparent     = 1;
  ProgressBar1.Caption         = ProgressBar1_Caption;
  ProgressBar1.FontName        = Tahoma19x19_Bold;
  ProgressBar1.Font_Color      = 0x0000;
  ProgressBar1.Gradient        = 1;
  ProgressBar1.Gradient_Orientation = 0;
  ProgressBar1.Gradient_Start_Color = 0xFFFF;
  ProgressBar1.Gradient_End_Color = 0x07E0;
  ProgressBar1.Color           = 0xA65E;
  ProgressBar1.Background_Color = 0x8410;
  ProgressBar1.Min             = 0;
  ProgressBar1.Max             = 100;
  ProgressBar1.Position        = 50;
  ProgressBar1.Show_Position   = 1;
  ProgressBar1.Show_Percentage = 1;
  ProgressBar1.Smooth          = 1;
  ProgressBar1.Rounded         = 1;
  ProgressBar1.Corner_Radius   = 1;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetCButton(index)             CurrentScreen->CButtons[index]
#define GetCRoundButton(index)        CurrentScreen->CButtons_Round[index]
#define GetCLabel(index)              CurrentScreen->CLabels[index]
#define GetProgressBar(index)         CurrentScreen->ProgressBars[index]

void DeleteTrailingSpaces(char* str){
  char counter;
  while(1) {
  if (str[0] == ' ') {
    for(counter = 0; counter < strlen(str); counter++) {
      str[counter] = str[counter+1];
    }
  }
  else
    break;
  }
}


void DrawCButton(TCButton *Abutton) {
  if (Abutton->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(Abutton->Transparent, Abutton->Press_Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_End_Color, Abutton->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    }
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Const_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    if (Abutton->TextAlign == _taLeft)
      TFT_Write_Const_Text(Abutton->Caption, Abutton->Left + 4, (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taCenter)
      TFT_Write_Const_Text(Abutton->Caption, (Abutton->Left + (Abutton->Width - caption_length) / 2), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
    else if (Abutton->TextAlign == _taRight)
      TFT_Write_Const_Text(Abutton->Caption, Abutton->Left + (Abutton->Width - caption_length - 4), (Abutton->Top + (Abutton->Height - caption_height) / 2));
  }
}

void DrawCRoundButton(TCButton_Round *Around_button) {
    if (Around_button->Visible == 1) {
      if (object_pressed == 1) {
        object_pressed = 0;
        TFT_Set_Brush(Around_button->Transparent, Around_button->Press_Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_End_Color, Around_button->Gradient_Start_Color);
      }
      else {
        TFT_Set_Brush(Around_button->Transparent, Around_button->Color, Around_button->Gradient, Around_button->Gradient_Orientation,
                      Around_button->Gradient_Start_Color, Around_button->Gradient_End_Color);
      }
      TFT_Set_Pen(Around_button->Pen_Color, Around_button->Pen_Width);
      TFT_Rectangle_Round_Edges(Around_button->Left + 1, Around_button->Top + 1,
        Around_button->Left + Around_button->Width - 2,
        Around_button->Top + Around_button->Height - 2, Around_button->Corner_Radius);
      TFT_Set_Font(Around_button->FontName, Around_button->Font_Color, FO_HORIZONTAL);
      TFT_Write_Const_Text_Return_Pos(Around_button->Caption, Around_button->Left, Around_button->Top);
      if (Around_button->TextAlign == _taLeft)
        TFT_Write_Const_Text(Around_button->Caption, Around_button->Left + 4, (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taCenter)
        TFT_Write_Const_Text(Around_button->Caption, (Around_button->Left + (Around_button->Width - caption_length) / 2), (Around_button->Top + ((Around_button->Height - caption_height) / 2)));
      else if (Around_button->TextAlign == _taRight)
        TFT_Write_Const_Text(Around_button->Caption, Around_button->Left + (Around_button->Width - caption_length - 4), (Around_button->Top + (Around_button->Height - caption_height) / 2));
    }
}

void DrawCLabel(TCLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Const_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void UpdatePBPosition(TProgressBar *AProgressBar) {
  int move_offset;
  unsigned long locPosition = 0;
  unsigned cnt = 0;
  char tmpStr[8];
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  move_offset = 10;
  locPosition = (unsigned long)(AProgressBar->Position - AProgressBar->Min) * 100 / (AProgressBar->Max - AProgressBar->Min);
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Background_Color, AProgressBar->Pen_Width);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    if (caption_height >  AProgressBar->Height)
        caption_height =  AProgressBar->Height;
    if (caption_length >  AProgressBar->Width)
        caption_length =  AProgressBar->Width;
    if (AProgressBar->Position < (AProgressBar->Left + (AProgressBar->Width - caption_length) / 2)) {
    TFT_Rectangle(AProgressBar->Left + (AProgressBar->Width - caption_length) / 2,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2,
                  AProgressBar->Left + (AProgressBar->Width - caption_length) / 2 + caption_length,
                  AProgressBar->Top +(AProgressBar->Height - caption_height) / 2 + caption_height);
    }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius  + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100 , AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      if (locPosition > 0) {
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
        while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
              if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
              else
          TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
          cnt = cnt + move_offset + 3;
        }
      }
    // Clear caption frist
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
    }
    else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
    }
    TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
    TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
    }
 }
}

void DrawProgressBar(TProgressBar *AProgressBar) {
  unsigned long locPosition = 0;
  int move_offset = 0;
  unsigned cnt = 0;
  unsigned int tmpWidth = 0;
  unsigned int tmpRound = 0;
  unsigned int _tmpI, _tmpII;
  char tmpStr[8];
  move_offset = 10;
  _tmpI = AProgressBar->Position - AProgressBar->Min;
  _tmpII = AProgressBar->Max - AProgressBar->Min;
  locPosition = (unsigned long)_tmpI * 100 / _tmpII;
  if (AProgressBar->Visible == 1) {
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
    TFT_Set_Pen(AProgressBar->Pen_Color, AProgressBar->Pen_Width);
    if (AProgressBar->Rounded == 1) {
      // First draw back rect
      TFT_Rectangle_Round_Edges(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height, AProgressBar->Corner_Radius);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left + AProgressBar->Corner_Radius, AProgressBar->Top,  AProgressBar->Left + AProgressBar->Corner_Radius   + locPosition *  (AProgressBar->Width-AProgressBar->Corner_Radius*2) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    else {
      // First draw back rect
      TFT_Rectangle(AProgressBar->Left, AProgressBar->Top, AProgressBar->Left + AProgressBar->Width, AProgressBar->Top + AProgressBar->Height);
      if (locPosition > 0) {
         // Then position rect
         TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Color, AProgressBar->Gradient, AProgressBar->Gradient_Orientation, AProgressBar->Gradient_Start_Color, AProgressBar->Gradient_End_Color);
         TFT_Rectangle(AProgressBar->Left, AProgressBar->Top,  AProgressBar->Left + (locPosition * AProgressBar->Width) / 100, AProgressBar->Top + AProgressBar->Height);
      }
    }
    // Draw not smoothed position rect
    if (AProgressBar->Smooth != 1) {
       if(AProgressBar->Rounded == 1) {
          tmpRound = AProgressBar->Corner_Radius;
          cnt = move_offset + tmpRound;
          tmpWidth = AProgressBar->Width - AProgressBar->Corner_Radius * 2;
       }
       else {
          tmpRound = 0;
          cnt = move_offset;
          tmpWidth = AProgressBar->Width;
       }
    TFT_Set_Brush(AProgressBar->Transparent, AProgressBar->Background_Color, 0,0,0,0);
     while (cnt-tmpRound < (locPosition * tmpWidth) / 100) {
            if  (cnt - tmpRound + 3 >=  tmpWidth * locPosition)
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top, AProgressBar->Left + tmpRound  + tmpWidth * locPosition , AProgressBar->Top + AProgressBar->Height);
            else
              TFT_Rectangle(AProgressBar->Left + cnt, AProgressBar->Top,  AProgressBar->Left + cnt + 3, AProgressBar->Top + AProgressBar->Height);
        cnt = cnt + move_offset + 3;
     }
    }
    // Clear caption first
    if (AProgressBar->Show_Position == 1){
       if (AProgressBar->Show_Percentage == 1) {
          WordToStr(locPosition , tmpStr);
          DeleteTrailingSpaces(tmpStr);
          strcpy(AProgressBar->Caption, tmpStr);
          strcat(AProgressBar->Caption, "\%");
       }
       else {
         WordToStr(AProgressBar->Position , tmpStr);
         DeleteTrailingSpaces(tmpStr);
         strcpy(AProgressBar->Caption, tmpStr);
       }
       TFT_Set_Font(AProgressBar->FontName, AProgressBar->Font_Color, FO_HORIZONTAL);
       TFT_Write_Text_Return_Pos(AProgressBar->Caption, AProgressBar->Left, AProgressBar->Top);
       TFT_Write_Text(AProgressBar->Caption, AProgressBar->Left + (AProgressBar->Width - caption_length) / 2, AProgressBar->Top + (AProgressBar->Height - caption_height) / 2);
     }
 }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short cbutton_idx;
  TCButton *local_cbutton;
  unsigned short round_cbutton_idx;
  TCButton_Round *local_round_cbutton;
  unsigned short clabel_idx;
  TCLabel *local_clabel;
  unsigned short progress_bar_idx;
  TProgressBar *local_progress_bar;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  cbutton_idx = 0;
  round_cbutton_idx = 0;
  clabel_idx = 0;
  progress_bar_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    save_bled_direction = TFT_BLED_Direction;
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 0, 1);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
    TFT_BLED_Direction = save_bled_direction;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (cbutton_idx < CurrentScreen->CButtonsCount) {
      local_cbutton = GetCButton(cbutton_idx);
      if (order == local_cbutton->Order) {
        cbutton_idx++;
        order++;
        DrawCButton(local_cbutton);
      }
    }

    if (round_cbutton_idx < CurrentScreen->CButtons_RoundCount) {
      local_round_cbutton = GetCRoundButton(round_cbutton_idx);
      if (order == local_round_cbutton->Order) {
        round_cbutton_idx++;
        order++;
        DrawCRoundButton(local_round_cbutton);
      }
    }

    if (clabel_idx < CurrentScreen->CLabelsCount) {
      local_clabel = GetCLabel(clabel_idx);
      if (order == local_clabel->Order) {
        clabel_idx++;
        order++;
        DrawCLabel(local_clabel);
      }
    }

    if (progress_bar_idx < CurrentScreen->ProgressBarsCount) {
      local_progress_bar = GetProgressBar(progress_bar_idx);
      if (order == local_progress_bar->Order) {
        progress_bar_idx++;
        order++;
        DrawProgressBar(local_progress_bar);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  cbutton_order       = -1;
  round_cbutton_order = -1;
  clabel_order        = -1;
  //  CButtons
  for ( _object_count = 0 ; _object_count < CurrentScreen->CButtonsCount ; _object_count++ ) {
    local_cbutton = GetCButton(_object_count);
    if (local_cbutton->Active == 1) {
      if (IsInsideObject(X, Y, local_cbutton->Left, local_cbutton->Top,
                         local_cbutton->Width, local_cbutton->Height) == 1) {
        cbutton_order = local_cbutton->Order;
        exec_cbutton = local_cbutton;
      }
    }
  }

  //  CButtons with Round Edges
  for ( _object_count = 0 ; _object_count < CurrentScreen->CButtons_RoundCount ; _object_count++ ) {
    local_round_cbutton = GetCRoundButton(_object_count);
    if (local_round_cbutton->Active == 1) {
      if (IsInsideObject(X, Y, local_round_cbutton->Left, local_round_cbutton->Top,
                         local_round_cbutton->Width, local_round_cbutton->Height) == 1) {
        round_cbutton_order = local_round_cbutton->Order;
        exec_round_cbutton = local_round_cbutton;
      }
    }
  }

  //  CLabels
  for ( _object_count = 0 ; _object_count < CurrentScreen->CLabelsCount ; _object_count++ ) {
    local_clabel = GetCLabel(_object_count);
    if (local_clabel->Active == 1) {
      if (IsInsideObject(X, Y, local_clabel->Left, local_clabel->Top,
                         local_clabel->Width, local_clabel->Height) == 1) {
        clabel_order = local_clabel->Order;
        exec_clabel = local_clabel;
      }
    }
  }

  _object_count = -1;
  if (cbutton_order >  _object_count )
    _object_count = cbutton_order;
  if (round_cbutton_order >  _object_count )
    _object_count = round_cbutton_order;
  if (clabel_order >  _object_count )
    _object_count = clabel_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_cbutton        = 0;
  exec_round_cbutton  = 0;
  exec_clabel         = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->OnPressPtr != 0) {
          exec_cbutton->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == round_cbutton_order) {
      if (exec_round_cbutton->Active == 1) {
        if (exec_round_cbutton->OnPressPtr != 0) {
          exec_round_cbutton->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnPressPtr != 0) {
          exec_clabel->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // CButton
    case 8: {
      if (PressedObject != 0) {
        exec_cbutton = (TCButton*)PressedObject;
        if ((exec_cbutton->PressColEnabled == 1) && (exec_cbutton->OwnerScreen == CurrentScreen)) {
          DrawCButton(exec_cbutton);
        }
        break;
      }
      break;
    }
    // Round CButton
    case 9: {
      if (PressedObject != 0) {
        exec_round_cbutton = (TCButton_Round*)PressedObject;
        if ((exec_round_cbutton->PressColEnabled == 1) && (exec_round_cbutton->OwnerScreen == CurrentScreen)) {
          DrawCRoundButton(exec_round_cbutton);
        }
        break;
      }
      break;
    }
  }

  exec_clabel          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // CButtons
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->OnUpPtr != 0)
          exec_cbutton->OnUpPtr();
        if (PressedObject == (TPointer)exec_cbutton)
          if (exec_cbutton->OnClickPtr != 0)
            exec_cbutton->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // CButtons with Round Edges
    if (_object_count == round_cbutton_order) {
      if (exec_round_cbutton->Active == 1) {
        if (exec_round_cbutton->OnUpPtr != 0)
          exec_round_cbutton->OnUpPtr();
        if (PressedObject == (TPointer)exec_round_cbutton)
          if (exec_round_cbutton->OnClickPtr != 0)
            exec_round_cbutton->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // CLabels
    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        if (exec_clabel->OnUpPtr != 0)
          exec_clabel->OnUpPtr();
        if (PressedObject == (TPointer)exec_clabel)
          if (exec_clabel->OnClickPtr != 0)
            exec_clabel->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_cbutton        = 0;
  exec_round_cbutton  = 0;
  exec_clabel         = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == cbutton_order) {
      if (exec_cbutton->Active == 1) {
        if (exec_cbutton->PressColEnabled == 1) {
          object_pressed = 1;
          DrawCButton(exec_cbutton);
        }
        PressedObject = (TPointer)exec_cbutton;
        PressedObjectType = 8;
        if (exec_cbutton->OnDownPtr != 0) {
          exec_cbutton->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == round_cbutton_order) {
      if (exec_round_cbutton->Active == 1) {
        if (exec_round_cbutton->PressColEnabled == 1) {
          object_pressed = 1;
          DrawCRoundButton(exec_round_cbutton);
        }
        PressedObject = (TPointer)exec_round_cbutton;
        PressedObjectType = 9;
        if (exec_round_cbutton->OnDownPtr != 0) {
          exec_round_cbutton->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == clabel_order) {
      if (exec_clabel->Active == 1) {
        PressedObject = (TPointer)exec_clabel;
        PressedObjectType = 10;
        if (exec_clabel->OnDownPtr != 0) {
          exec_clabel->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  PLLEN_bit = 1;
  Delay_ms(150);
  WDTCON.B4 = 1;
  ANCON0 = 0xF0; // All pins to digital
  ANCON1 = 0xFF;
  WDTCON.B4 = 0;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(96, 766, 99, 754);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
